% This code reproduces experimental results from the paper:
%
% A. G. Savi, S. ivkovi, K. K. Jovanovi, L. Duponchel, I. Kopriva (2015).
% Complete determination of plant tissues and advanced image analysis
% - study of needles and stamen. Journal of Chemometrics,doi: 10.1002/cem/2735 
%

clear;
close all;

% J_Chemometrics_2015_Savic
                       
image_type = 2;        % 1 - needle (Figure 2.1);  2 - stamen (Figure 2.2);  
  
%% read multispectral image

if image_type == 1
    cd .\iglica
    M1 = imread('01bf','jpg');
    [I1 I2] = size(M1);
    X(1,:) = double(reshape(M1,1,I1*I2));
    M2 = imread('01dapi','jpg');
    X(2,:) = double(reshape(M2,1,I1*I2));
    M3 = imread('01fam','jpg');
    X(3,:) = double(reshape(M3,1,I1*I2));
    M4 = imread('01gfp','jpg');
    X(4,:) = double(reshape(M4,1,I1*I2));
    M5 = imread('01red','jpg');
    X(5,:) = double(reshape(M5,1,I1*I2));
elseif image_type == 2
    cd .\polen
    M1 = imread('01bf','jpg');
    [I1 I2] = size(M1);
    X(1,:) = double(reshape(M1,1,I1*I2));
    M2 = imread('01dapi','jpg');
    X(2,:) = double(reshape(M2,1,I1*I2));
    M3 = imread('01fam','jpg');
    X(3,:) = double(reshape(M3,1,I1*I2));
    M4 = imread('01gfp','jpg');
    X(4,:) = double(reshape(M4,1,I1*I2));
    M5 = imread('01red','jpg');
    X(5,:) = double(reshape(M5,1,I1*I2));    
end

%% display gray scale spectral images
figure(1)
subplot(2,3,1);
colormap(gray)
imagesc(M1);
set(gca,'Xtick',[]);
set(gca,'Ytick',[]);
title('Mixed image - M1')
subplot(2,3,2);
colormap(gray)
imagesc(M2);
set(gca,'Xtick',[]);
set(gca,'Ytick',[]);
title('Mixed image - M2')
subplot(2,3,3);
colormap(gray)
imagesc(M3);
set(gca,'Xtick',[]);
set(gca,'Ytick',[]);
title('Mixed image - M3')
subplot(2,3,4);
colormap(gray)
imagesc(M4);
set(gca,'Xtick',[]);
set(gca,'Ytick',[]);
title('Mixed image - M4')
subplot(2,3,5);
colormap(gray)
imagesc(M5);
set(gca,'Xtick',[]);
set(gca,'Ytick',[]);
title('Mixed image - M5')

clear M1 M2 M3 M4 M5

% scale multispectral image to [0, 1] range
xm = max(max(X));
X = X/xm;

%% use approximate Gaussian explicite feature map with order=2 and stdev=1
NS = 5;
stdev=1;
order=2;
FIX = feature_map(X,'gaussian',stdev,order);   

%% call the NMF_L0 method
 xmin = min(min(FIX));
 if xmin < 0
     display('Negative entried are not allowed by NMF_L0 algorithm')
     display('They will be set to zero')
     FIX = max(FIX,0);
 end
  
 %% set NMFL0_H parameters
        options.K = size(FIX,1); % number of sources/components
        options.L = 1;           % number of overlapping sources
        options.numIter = 7;     % number of iterations

        %%% select sparse coder
        options.sparseCoder = @rsNNLS___;
        % options.sparseCoder = @sNNLS___;

        %%% select dictionary update method
        options.updateType = 'ANLS_FC';
        % options.updateType = 'MU';
        % options.updateType = 'NNKSVD';
        % options.updateType = 'ANLS_PG';
        % options.NNLS_PG_tolerance = 1e-3;
        % options.NNLS_PG_maxIter = 100;

        %%% number of dictionary updates
        options.numUpdateIter = 10;
        [AH,SH,INFO] = NMFL0_H(X,options);
        
figure
for i=1:size(FIX,1)
    figure
   % subplot(5,5,i)
  %  colormap(gray)
    imagesc(reshape(SH(i,:),I1,I2))
    set(gca,'Xtick',[]);
    set(gca,'Ytick',[]);
    title(strcat('EFM_2_NMF_L0_SH',num2str(i)))
    SHi=reshape(SH(i,:),I1,I2);
    SHi=mat2gray(SHi);
   % imwrite(SHi, strcat('SH',num2str(i),'_EFM_2_sig2_0p1_NMF_L0.tiff'), 'tiff', 'Compression', 'none');
end
